@@ -306,11 +306,6 @@ HEADER_INLINE void memcpyx(void* __restrict dst, const void* __restrict src, uin
   S_CAST(char*, dst)[ct] = extra_char;
 }
 
-HEADER_INLINE char* strcpya(char* __restrict dst, const void* __restrict src) {
-  const uintptr_t slen = strlen(S_CAST(const char*, src));
-  return memcpya(dst, src, slen);
-}
-
 HEADER_INLINE char* strcpyax(char* __restrict dst, const void* __restrict src, char extra_char) {
   const uintptr_t slen = strlen(S_CAST(const char*, src));
   memcpy(dst, src, slen);
@@ -345,7 +340,8 @@ template <uint32_t N> int32_t StrequalK(const char* s1, const char* k_s2, uint32
   return (s1_slen == N) && memequal_k(s1, k_s2, N);
 };
 
-// CompileTimeSlen and strcpy_k moved to plink2_base
+// strcpya, CompileTimeSlen, strcpy_k, strcpya_k, u32toa, i64toa, wtoa moved to
+// plink2_base
 
 // can also use sizeof(k_s2) - 1, but that's less safe
 #  define strequal_k(s1, k_s2, s1_slen) plink2::StrequalK<plink2::CompileTimeSlen(k_s2)>(s1, k_s2, s1_slen)
@@ -354,7 +350,6 @@ template <uint32_t N> int32_t StrequalK(const char* s1, const char* k_s2, uint32
 
 #  define memequal_sk(s1, k_s2) memequal_k(s1, k_s2, plink2::CompileTimeSlen(k_s2))
 
-#  define strcpya_k(dst, src) plink2::MemcpyaoK<plink2::CompileTimeSlen(src)>(dst, src);
 #else  // !(defined(__LP64__) && (__cplusplus >= 201103L))
 HEADER_INLINE char* memcpyax_k(void* __restrict dst, const void* __restrict src, uint32_t ct, char extra_char) {
   return memcpyax(dst, src, ct, extra_char);
@@ -378,10 +373,6 @@ HEADER_INLINE int32_t memequal_sk(const void* s1, const char* k_s2) {
   const uint32_t s2_slen = strlen(k_s2);
   return memequal(s1, k_s2, s2_slen);
 }
-
-HEADER_INLINE char* strcpya_k(char* __restrict dst, const void* __restrict src) {
-  return strcpya(dst, src);
-}
 #endif
 
 // Must be safe to read first slen bytes of unknown_len_str.
@@ -469,6 +460,7 @@ HEADER_INLINE int32_t StrEndsWith(const char* s_read, const char* s_suffix_const
 
 // May read (kBytesPerWord - 1) bytes past the end of each string.
 HEADER_INLINE int32_t strequal_overread(const char* s1, const char* s2) {
+#ifndef NO_UNALIGNED
   const uintptr_t* s1_alias = R_CAST(const uintptr_t*, s1);
   const uintptr_t* s2_alias = R_CAST(const uintptr_t*, s2);
   for (uintptr_t widx = 0; ; ++widx) {
@@ -485,6 +477,9 @@ HEADER_INLINE int32_t strequal_overread(const char* s1, const char* s2) {
       return 0;
     }
   }
+#else
+  return (strcmp(s1, s2) == 0);
+#endif
 }
 
 int32_t strcmp_overread(const char* s1, const char* s2);
@@ -541,6 +536,7 @@ typedef struct StrSortDerefStruct {
 } StrSortDeref;
 
 HEADER_INLINE bool strcmp_overread_lt(const char* s1, const char* s2) {
+#  ifndef NO_UNALIGNED
   const uintptr_t* s1_alias = R_CAST(const uintptr_t*, s1);
   const uintptr_t* s2_alias = R_CAST(const uintptr_t*, s2);
   for (uintptr_t widx = 0; ; ++widx) {
@@ -566,13 +562,16 @@ HEADER_INLINE bool strcmp_overread_lt(const char* s1, const char* s2) {
     }
     if (w1 != w2) {
     strcmp_overread_lt_finish:
-#  ifdef __LP64__
+#    ifdef __LP64__
       return __builtin_bswap64(w1) < __builtin_bswap64(w2);
-#  else
+#    else
       return __builtin_bswap32(w1) < __builtin_bswap32(w2);
-#  endif
+#    endif
     }
   }
+#  else // NO_UNALIGNED
+  return strcmp(s1, s2) < 0;
+#  endif
 }
 
 typedef struct StrSortDerefOverreadStruct {
@@ -1385,30 +1384,12 @@ uint32_t CountTokens(const char* str_iter);
 // empty multistr ok
 uint32_t CountAndMeasureMultistr(const char* multistr, uintptr_t* max_blen_ptr);
 
-extern const uint16_t kDigitPair[];
-
-char* u32toa(uint32_t uii, char* start);
-
 char* i32toa(int32_t ii, char* start);
 
 char* u32toa_z5(uint32_t uii, char* start);
 
 char* u32toa_z6(uint32_t uii, char* start);
 
-char* i64toa(int64_t llii, char* start);
-
-#ifdef __LP64__
-// really just for printing line numbers
-// must be less than 2^63
-HEADER_INLINE char* wtoa(uintptr_t ulii, char* start) {
-  return i64toa(ulii, start);
-}
-#else
-HEADER_INLINE char* wtoa(uintptr_t ulii, char* start) {
-  return u32toa(ulii, start);
-}
-#endif
-
 char* u32toa_trunc4(uint32_t uii, char* start);
 
 // Write-buffer-sizing constants, to support replacement of dtoa_g() with
@@ -1536,9 +1517,6 @@ HEADER_INLINE BoolErr SortedIdboxFind(const char* idbuf, const char* sorted_idbo
   return 0;
 }
 
-#ifdef NO_UNALIGNED
-#  error "Unaligned accesses in IsNanStr()."
-#endif
 // This returns 1 on any capitalization of 'na' or 'nan', 0 otherwise.
 // todo: check whether there's actually any point to the uint16_t type-pun
 HEADER_INLINE uint32_t IsNanStr(const char* ss, uint32_t slen) {
@@ -1548,7 +1526,13 @@ HEADER_INLINE uint32_t IsNanStr(const char* ss, uint32_t slen) {
   if (!slen) {
     return 1;
   }
+#ifndef NO_UNALIGNED
   const uint32_t first_two_chars_code = R_CAST(const uint16_t*, ss)[0];
+#else
+  uint16_t first_two_chars_code_u16;
+  memcpy_k(&first_two_chars_code_u16, ss, 2);
+  const uint32_t first_two_chars_code = first_two_chars_code_u16;
+#endif
   // assumes little-endian
   if ((first_two_chars_code & 0xdfdf) != 0x414e) {
     return 0;
